/**********************************************************
 * Filename:    FSM_CRS.h
 *
 * Description: Carrier Status state machine
 *
 *********************************************************/


#ifndef _FSM_CRS_H_INCLUDED_
#define _FSM_CRS_H_INCLUDED_

#define CARRIER_ON  true
#define CARRIER_OFF false

const int32s BLOCK_BYTES    = 62; // Note changed value (was 54)
const int32s PARITY_RATIO   = 8;

#include "FSM_base.h"

class fsm_crs_generation_t: public fsm_base_t< DLY_CRS_GEN, _36b_t >
{
private:
    bool    CARRIER_STATUS;
    int32s  tx_cnt;
    int32s  parity_cnt;

public:

    fsm_crs_generation_t()
    {
        CARRIER_STATUS = CARRIER_OFF;
        tx_cnt = PARITY_RATIO; 
        parity_cnt = 0;
    }
    /////////////////////////////////////////////////////////////
    // 
    /////////////////////////////////////////////////////////////
    bool GetCRS( void )
    {
        //return false;   // disable carrier sense generation
        return CARRIER_STATUS;
    }

    /////////////////////////////////////////////////////////////
    // 
    /////////////////////////////////////////////////////////////
    void  ReceiveUnit( _36b_t col )
    {
        tx_cnt++;

        if( tx_cnt == BLOCK_BYTES )
        {
            parity_cnt += PARITY_RATIO;
            tx_cnt = 0;
        }

        if( parity_cnt > 0 && CARRIER_STATUS == CARRIER_ON && col.IsType( C_BLOCK ))
            parity_cnt--;

        CARRIER_STATUS = parity_cnt > 0? CARRIER_ON: CARRIER_OFF;

        output_block = col;
        output_ready = true;
    }

};

#endif // _FSM_CRS_H_INCLUDED_