
/**********************************************************
 * Filename:    FSM_II.h
 *
 * Description: Defines Idle Insertion process
 *
 *********************************************************/


#ifndef _FSM_IDLE_INSERTION_H_INCLUDED_
#define _FSM_IDLE_INSERTION_H_INCLUDED_

#include "_queue.h"

#define IDLE_VECTOR _72b_t( 0, C_BLOCK )

const int32s FIFO_II_SIZE  = BLK_ROUNDUP( MAX_FRAME_BYTES, VECTOR_BYTES )
						   + BLK_ROUNDUP( MAX_FRAME_BYTES, FEC_PAYLOAD_BYTES ) * FEC_PSIZE;

class fsm_idle_insertion_t: public fsm_base_t< DLY_IDLE_INS, _72b_t >
{
private:
    Queue< _72b_t, FIFO_II_SIZE > FIFO_II;
	
	int32s  FrameReadyCount;
	int32s  RxVectorCount;
    int32s  ExcessIdleCount;

    /////////////////////////////////////////////////////////////
    // 
    /////////////////////////////////////////////////////////////
    void ReceiveUnit( _72b_t vector )
    {
		if( FIFO_II.IsFull() )
		{
			MSG_WARN( "Attempt to receive into a full FIFO_II" );
			return;
		}

		FIFO_II.Add( vector );

		if( vector.IsType( T_BLOCK ))
			FrameReadyCount++;
    }

	/////////////////////////////////////////////////////////////
    // 
    /////////////////////////////////////////////////////////////
    _72b_t TransmitUnit( void )
    {
		if( FIFO_II.IsEmpty() )
		{
				MSG_WARN( "Attempt to transmit from an empty FIFO_II" );
				exit(0); 
		}

		blk_t block_type = FIFO_II.Peek(0).T_TYPE();
       
		if( ( block_type == S_BLOCK && FrameReadyCount == 0 ) ||
			( block_type != D_BLOCK && block_type != T_BLOCK && ExcessIdleCount < 0 ))
		{
			ExcessIdleCount++;
			return IDLE_VECTOR;
		}
		else
		{
			if( FrameReadyCount <= 0 && block_type != C_BLOCK )
				MSG_WARN( "Idle Insertion process sends partially-received frame" );

			RxVectorCount ++;
			if( RxVectorCount == FEC_DSIZE )
			{
				RxVectorCount = 0;	
				ExcessIdleCount -= FEC_PSIZE;
			}

			if( block_type == T_BLOCK )
				FrameReadyCount--;

			return FIFO_II.Get();
		}
    }


public:
    fsm_idle_insertion_t()
    {
		FrameReadyCount = 0;
		RxVectorCount   = 0;
		ExcessIdleCount = 0;

		for( int32s index = 0; index < 2 * FEC_DSIZE; index++ )
		{
			FIFO_II.Add( _72b_t( 0, C_BLOCK ) );
		}
    }

    
};

#endif // _FSM_IDLE_INSERTION_H_INCLUDED_
