/**********************************************************
 * Filename:    FSM_MPCP.h
 *
 * Description: 
 *
 *********************************************************/


#ifndef _FSM_MPCP_H_INCLUDED_
#define _FSM_MPCP_H_INCLUDED_

#include "FSM_base.h"

/////////////////////////////////////////////////////////////////////
#define MSDU_BYTES( L )   (( L ) - E_HEADER_STRING_BYTES - CHECKSUM_BYTES )

/////////////////////////////////////////////////////////////////////
// MPCP TX state machine 
/////////////////////////////////////////////////////////////////////
template< int16s (*pf_packet_size)( void ) > class fsm_mpcp_tx_t: public fsm_base_t< DLY_MPCP_TX, _frm_t >
{
    private:
        int32s  frame_count; 
		clk_t   initiate_timer;
        bool    mac_ready;
        bool    burst_mode;

        /////////////////////////////////////////////////////////////
        //  
        /////////////////////////////////////////////////////////////
        clk_t FEC_Overhead_Min( int16s length )
        {
			return ( length / FEC_PAYLOAD_BYTES ) * FEC_PARITY_BYTES;
        }

        /////////////////////////////////////////////////////////////
        //  
        /////////////////////////////////////////////////////////////
        void ReceiveUnit( _frm_t ) {}

        /////////////////////////////////////////////////////////////
        //  
        /////////////////////////////////////////////////////////////
        _frm_t TransmitUnit( void )
        {
            if( !output_ready )
            {
                MSG_WARN( "MPCP Frame is not ready" );
                exit(0);
            }
          
			output_ready = false;
			mac_ready    = false;

            int16s frame_size = pf_packet_size() + PREAMBLE_BYTES;
            initiate_timer = FEC_Overhead_Min( frame_size + MIN_IPG_BYTES ) + MIN_IPG_BYTES; 

            if( burst_mode && ++frame_count == BURST_FRAMES )
            {
                frame_count = 0;
                initiate_timer = BURST_GAP_BYTES;
            }

            return _frm_t( timestamp_t::GetClock(), frame_size );
        }


    public:
        /////////////////////////////////////////////////////////////
        //  
        /////////////////////////////////////////////////////////////
        fsm_mpcp_tx_t( bool brst_md = false )
        {
            burst_mode     = brst_md;
            frame_count    = 0;
            mac_ready      = true;
            initiate_timer = burst_mode? BURST_GAP_BYTES : 0;
        }

        /////////////////////////////////////////////////////////////
        //  
        /////////////////////////////////////////////////////////////
		inline void SignalMacReady( bool mac_rdy ) { mac_ready = mac_rdy; }

        /////////////////////////////////////////////////////////////
        //  
        /////////////////////////////////////////////////////////////
        inline void IncrementByteClock( void ) 
		{
			if( mac_ready && initiate_timer > 0 )
				initiate_timer--;

			output_ready = mac_ready && (initiate_timer == 0);
		}
};

/////////////////////////////////////////////////////////////////////
// MPCP RX state machine 
/////////////////////////////////////////////////////////////////////
class fsm_mpcp_rx_t: public fsm_base_t< DLY_MPCP_RX, _frm_t >
{
    private:
        /////////////////////////////////////////////////////////////
        void ReceiveUnit( _frm_t in_blk )
        {
            output_block = in_blk;
            output_ready = true;
        }
};

#endif //_FSM_MPCP_H_INCLUDED_